import * as docx from 'docx';
import { IPropertiesOptions, IRunStylePropertiesOptions, IParagraphStylePropertiesOptions, Document, FileChild, ParagraphChild, Paragraph } from 'docx';
export { Packer } from 'docx';
import { Token, MarkedOptions, Tokens } from 'marked';

/**
 * Represents a single footnote.
 */
type Footnote = {
    id: number;
    type: 'footnote';
    raw: string;
    label: string;
    tokens: Token[];
};
/**
 * Represents a reference to a footnote.
 */
type FootnoteRef = {
    type: 'footnoteRef';
    raw: string;
    id: number;
    label: string;
};

type MarkdownImageType = 'jpg' | 'png' | 'gif' | 'bmp' | 'webp';
type MarkdownImageItem = {
    type: MarkdownImageType;
    data: Buffer | string | Uint8Array | ArrayBuffer;
    width: number;
    height: number;
};
type MarkdownImageAdapter = (token: Tokens.Image) => Promise<null | MarkdownImageItem>;
interface MarkdownDocxOptions extends MarkedOptions {
    imageAdapter?: MarkdownImageAdapter;
    /**
     * do not download image
     * @default false
     */
    ignoreImage?: boolean;
    /**
     * do not parse footnote
     * @default false
     */
    ignoreFootnote?: boolean;
    /**
     * do not parse html
     * @default false
     */
    ignoreHtml?: boolean;
    /**
     * Properties for the document
     */
    document?: Omit<IPropertiesOptions, 'sections'>;
}
type IBlockToken = Tokens.Space | Tokens.Code | Tokens.Heading | Tokens.Hr | Tokens.Blockquote | Tokens.List | Tokens.HTML | Tokens.Def | Tokens.Table | Tokens.Heading | Tokens.Paragraph | Tokens.Text | Footnote;
type IInlineToken = Tokens.Escape | Tokens.Tag | Tokens.Link | Tokens.Em | Tokens.Strong | Tokens.Codespan | Tokens.Br | Tokens.Del | Tokens.Text | Tokens.Image | FootnoteRef;
type IParagraphToken = Tokens.Paragraph | Tokens.Blockquote | Tokens.Heading;
type ITextAttr = {
    style?: string;
    bold?: boolean;
    italics?: boolean;
    underline?: boolean;
    strike?: boolean;
    break?: boolean | number;
    html?: boolean;
    link?: boolean;
    strong?: boolean;
    em?: boolean;
    codespan?: boolean;
    del?: boolean;
    br?: boolean;
};
type IBlockAttr = {
    style?: string;
    blockquote?: boolean;
    list?: {
        task?: boolean;
        checked?: boolean;
        level: number;
        type?: 'number' | 'bullet';
    };
    heading?: number;
    code?: boolean;
    align?: 'left' | 'center' | 'right' | null;
    footnote?: boolean;
};
type Writeable<T> = {
    -readonly [P in keyof T]: T[P];
};
type IMarkdownToken = 'space' | 'code' | 'hr' | 'blockquote' | 'html' | 'def' | 'paragraph' | 'text' | 'footnote' | 'listItem' | 'table' | 'tableHeader' | 'tableCell' | 'heading1' | 'heading2' | 'heading3' | 'heading4' | 'heading5' | 'heading6' | 'tag' | 'link' | 'strong' | 'em' | 'codespan' | 'del' | 'br';
type IMarkdownStyle = {
    inline?: boolean;
    className: string;
    name?: string;
    basedOn?: string;
    next?: string;
    run?: IRunStylePropertiesOptions;
    paragraph?: IParagraphStylePropertiesOptions;
    quickFormat?: boolean;
    properties?: any;
};

declare class MarkdownDocx {
    markdown: string;
    options: MarkdownDocxOptions;
    static defaultOptions: MarkdownDocxOptions;
    styles: {
        colors: {
            primary: string;
            secondary: string;
            accent1: string;
            accent2: string;
            accent3: string;
            heading1: string;
            heading2: string;
            heading3: string;
            link: string;
            code: string;
            blockquote: string;
            del: string;
        };
        classes: {
            readonly Space: "MdSpace";
            readonly Code: "MdCode";
            readonly Hr: "MdHr";
            readonly Blockquote: "MdBlockquote";
            readonly Html: "MdHtml";
            readonly Def: "MdDef";
            readonly Paragraph: "MdParagraph";
            readonly Text: "MdText";
            readonly Footnote: "MdFootnote";
            readonly ListItem: "MdListItem";
            readonly Table: "MdTable";
            readonly TableHeader: "MdTableHeader";
            readonly TableCell: "MdTableCell";
            readonly Heading1: "MdHeading1";
            readonly Heading2: "MdHeading2";
            readonly Heading3: "MdHeading3";
            readonly Heading4: "MdHeading4";
            readonly Heading5: "MdHeading5";
            readonly Heading6: "MdHeading6";
            readonly Tag: "MdTag";
            readonly Link: "MdLink";
            readonly Strong: "MdStrong";
            readonly Em: "MdEm";
            readonly Codespan: "MdCodespan";
            readonly Del: "MdDel";
            readonly Br: "MdBr";
        };
        default: {
            readonly document?: docx.IDocumentDefaultsOptions;
            readonly title?: docx.IBaseParagraphStyleOptions;
            readonly heading1?: docx.IBaseParagraphStyleOptions;
            readonly heading2?: docx.IBaseParagraphStyleOptions;
            readonly heading3?: docx.IBaseParagraphStyleOptions;
            readonly heading4?: docx.IBaseParagraphStyleOptions;
            readonly heading5?: docx.IBaseParagraphStyleOptions;
            readonly heading6?: docx.IBaseParagraphStyleOptions;
            readonly strong?: docx.IBaseParagraphStyleOptions;
            readonly listParagraph?: docx.IBaseParagraphStyleOptions;
            readonly hyperlink?: docx.IBaseCharacterStyleOptions;
            readonly footnoteReference?: docx.IBaseCharacterStyleOptions;
            readonly footnoteText?: docx.IBaseParagraphStyleOptions;
            readonly footnoteTextChar?: docx.IBaseCharacterStyleOptions;
        } | undefined;
        markdown: Record<IMarkdownToken, IMarkdownStyle>;
        numbering: docx.INumberingOptions;
    };
    static covert(markdown: string, _options?: MarkdownDocxOptions): Promise<Document>;
    protected _imageStore: Map<string, MarkdownImageItem>;
    private footnotes;
    constructor(markdown: string, options?: MarkdownDocxOptions);
    get ignoreImage(): boolean;
    get ignoreFootnote(): boolean;
    get ignoreHtml(): boolean;
    toDocument(options?: Omit<IPropertiesOptions, 'sections'>): Promise<Document>;
    toSection(): Promise<FileChild[]>;
    downloadImageList(tokens: Tokens.Image[]): Promise<(MarkdownImageItem | undefined)[]>;
    toBlocks(tokens: IBlockToken[], attr?: IBlockAttr): FileChild[];
    toTexts(tokens: IInlineToken[], attr?: ITextAttr): ParagraphChild[];
    addFootnote(id: number, children: Paragraph[]): void;
    findImage(token: Tokens.Image): MarkdownImageItem | null;
}

declare const styles: {
    colors: {
        primary: string;
        secondary: string;
        accent1: string;
        accent2: string;
        accent3: string;
        heading1: string;
        heading2: string;
        heading3: string;
        link: string;
        code: string;
        blockquote: string;
        del: string;
    };
    classes: {
        readonly Space: "MdSpace";
        readonly Code: "MdCode";
        readonly Hr: "MdHr";
        readonly Blockquote: "MdBlockquote";
        readonly Html: "MdHtml";
        readonly Def: "MdDef";
        readonly Paragraph: "MdParagraph";
        readonly Text: "MdText";
        readonly Footnote: "MdFootnote";
        readonly ListItem: "MdListItem";
        readonly Table: "MdTable";
        readonly TableHeader: "MdTableHeader";
        readonly TableCell: "MdTableCell";
        readonly Heading1: "MdHeading1";
        readonly Heading2: "MdHeading2";
        readonly Heading3: "MdHeading3";
        readonly Heading4: "MdHeading4";
        readonly Heading5: "MdHeading5";
        readonly Heading6: "MdHeading6";
        readonly Tag: "MdTag";
        readonly Link: "MdLink";
        readonly Strong: "MdStrong";
        readonly Em: "MdEm";
        readonly Codespan: "MdCodespan";
        readonly Del: "MdDel";
        readonly Br: "MdBr";
    };
    default: {
        readonly document?: docx.IDocumentDefaultsOptions;
        readonly title?: docx.IBaseParagraphStyleOptions;
        readonly heading1?: docx.IBaseParagraphStyleOptions;
        readonly heading2?: docx.IBaseParagraphStyleOptions;
        readonly heading3?: docx.IBaseParagraphStyleOptions;
        readonly heading4?: docx.IBaseParagraphStyleOptions;
        readonly heading5?: docx.IBaseParagraphStyleOptions;
        readonly heading6?: docx.IBaseParagraphStyleOptions;
        readonly strong?: docx.IBaseParagraphStyleOptions;
        readonly listParagraph?: docx.IBaseParagraphStyleOptions;
        readonly hyperlink?: docx.IBaseCharacterStyleOptions;
        readonly footnoteReference?: docx.IBaseCharacterStyleOptions;
        readonly footnoteText?: docx.IBaseParagraphStyleOptions;
        readonly footnoteTextChar?: docx.IBaseCharacterStyleOptions;
    } | undefined;
    markdown: Record<IMarkdownToken, IMarkdownStyle>;
    numbering: docx.INumberingOptions;
};

declare function markdownDocx(markdown: string, options?: MarkdownDocxOptions): Promise<docx.Document>;

export { type IBlockAttr, type IBlockToken, type IInlineToken, type IMarkdownStyle, type IMarkdownToken, type IParagraphToken, type ITextAttr, MarkdownDocx, type MarkdownDocxOptions, type MarkdownImageAdapter, type MarkdownImageItem, type MarkdownImageType, type Writeable, markdownDocx as default, markdownDocx, styles };
